==> picolibc_1.8.9.orig.tar.gz: picolibc-1.8.9/semihost/machine/lm32/lm32_scall.c <==
/*
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright © 2024 Keith Packard
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "lm32_semihost.h"

void
lm32_scall(const struct lm32_scall_args *args,
           struct lm32_scall_ret *ret)
{
    register uint32_t r8 __asm__("r8") = args->r8;
    register uint32_t r1 __asm__("r1") = args->r1;
    register uint32_t r2 __asm__("r2") = args->r2;
    register uint32_t r3 __asm__("r3") = args->r3;
    register uint32_t r4 __asm__("r4") = args->r4;

    __asm__("scall" : "=r" (r1), "=r" (r2), "=r" (r3) :
            "r" (r8), "r" (r1), "r" (r2), "r" (r3), "r" (r4));

    ret->r1 = r1;
    ret->r2 = r2;
    ret->r3 = r3;
    if ((int) ret->r1 < 0)
        lm32_errno(ret->r3);
}

==> picolibc_1.8.11.orig.tar.gz: picolibc-1.8.11/semihost/machine/lm32/lm32_scall.c <==
/*
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright © 2024 Keith Packard
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "lm32_semihost.h"

void
lm32_scall(const struct lm32_scall_args *args, struct lm32_scall_ret *ret)
{
    register uint32_t r8 __asm__("r8") = args->r8;
    register uint32_t r1 __asm__("r1") = args->r1;
    register uint32_t r2 __asm__("r2") = args->r2;
    register uint32_t r3 __asm__("r3") = args->r3;
    register uint32_t r4 __asm__("r4") = args->r4;

    __asm__("scall" : "=r"(r1), "=r"(r2), "=r"(r3) : "r"(r8), "r"(r1), "r"(r2), "r"(r3), "r"(r4));

    ret->r1 = r1;
    ret->r2 = r2;
    ret->r3 = r3;
    if ((int)ret->r1 < 0)
        lm32_errno(ret->r3);
}
